@@ -1,21 +1,9 @@
-//
-// Copyright (C) 2016~2016 by CSSlayer
-// wengxt@gmail.com
-//
-// This library is free software; you can redistribute it and/or modify
-// it under the terms of the GNU Lesser General Public License as
-// published by the Free Software Foundation; either version 2.1 of the
-// License, or (at your option) any later version.
-//
-// This library is distributed in the hope that it will be useful,
-// but WITHOUT ANY WARRANTY; without even the implied warranty of
-// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
-// Lesser General Public License for more details.
-//
-// You should have received a copy of the GNU Lesser General Public
-// License along with this library; see the file COPYING. If not,
-// see <http://www.gnu.org/licenses/>.
-//
+/*
+ * SPDX-FileCopyrightText: 2016-2016 CSSlayer <wengxt@gmail.com>
+ *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ *
+ */
 
 #ifndef _FCITX_INPUTCONTEXT_H_
 #define _FCITX_INPUTCONTEXT_H_
@@ -51,6 +39,7 @@ typedef std::function<bool(InputContext *ic)> InputContextVisitor;
 class FCITXCORE_EXPORT InputContext : public TrackableObject<InputContext> {
     friend class InputContextManagerPrivate;
     friend class FocusGroup;
+    friend class UserInterfaceManager;
 
 public:
     InputContext(InputContextManager &manager, const std::string &program = {});
@@ -72,7 +61,7 @@ public:
     void setCapabilityFlags(CapabilityFlags flags);
     CapabilityFlags capabilityFlags() const;
     void setCursorRect(Rect rect);
-    bool keyEvent(KeyEvent &key);
+    bool keyEvent(KeyEvent &event);
 
     bool hasFocus() const;
 
@@ -84,9 +73,22 @@ public:
     void deleteSurroundingText(int offset, unsigned int size);
     void forwardKey(const Key &rawKey, bool isRelease = false, int time = 0);
     void updatePreedit();
-    void updateUserInterface(UserInterfaceComponent componet,
+    void updateUserInterface(UserInterfaceComponent component,
                              bool immediate = false);
 
+    /**
+     * Prevent event deliver to input context, and re-send the event later.
+     *
+     * This should be only used by frontend to make sync and async event handled
+     * in the same order.
+     *
+     * @param block block state of input context.
+     *
+     * @see InputContextEventBlocker
+     */
+    void setBlockEventToClient(bool block);
+    bool hasPendingEvents() const;
+
     InputContextProperty *property(const std::string &name);
     InputContextProperty *property(const InputContextPropertyFactory *factory);
 
@@ -111,6 +113,7 @@ protected:
     virtual void deleteSurroundingTextImpl(int offset, unsigned int size) = 0;
     virtual void forwardKeyImpl(const ForwardKeyEvent &key) = 0;
     virtual void updatePreeditImpl() = 0;
+    virtual void updateClientSideUIImpl();
     void destroy();
     void created();
 
@@ -120,6 +123,16 @@ private:
     std::unique_ptr<InputContextPrivate> d_ptr;
     FCITX_DECLARE_PRIVATE(InputContext);
 };
+
+class FCITXCORE_EXPORT InputContextEventBlocker {
+public:
+    InputContextEventBlocker(InputContext *inputContext);
+    ~InputContextEventBlocker();
+
+private:
+    TrackableObjectReference<InputContext> inputContext_;
+};
+
 } // namespace fcitx
 
 #endif // _FCITX_INPUTCONTEXT_H_
